// g2o - General Graph Optimization
// Copyright (C) 2011 R. Kuemmerle, G. Grisetti, W. Burgard
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// * Redistributions of source code must retain the above copyright notice,
//   this list of conditions and the following disclaimer.
// * Redistributions in binary form must reproduce the above copyright
//   notice, this list of conditions and the following disclaimer in the
//   documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
// IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
// TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
// TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "edge_2DOF_prior.h"
#include <iostream>

namespace g2o {
  using namespace std;

  // point to camera projection, monocular
  MyEdge2DOFPrior::MyEdge2DOFPrior() : BaseUnaryEdge<2, Eigen::Vector2d, Vertex2DOF>() {
    setMeasurement(Eigen::Vector2d::Zero());
    information().setIdentity();
    resizeParameters(1);
  }

  bool MyEdge2DOFPrior::read(std::istream& is)
  {
    Vector2d p;
    is >> p[0] >> p[1];
    setMeasurement(p);
    for (int i = 0; i < 2; ++i)
      for (int j = i; j < 2; ++j) {
        is >> information()(i, j);
        if (i != j)
          information()(j, i) = information()(i, j);
      }
    return true;
  }

  bool MyEdge2DOFPrior::write(std::ostream& os) const
  {
    Vector2d p = measurement();
    os << p.x() << " " << p.y();
    for (int i = 0; i < 2; ++i)
      for (int j = i; j < 2; ++j)
        os << " " << information()(i, j);
    return os.good();
  }

  void MyEdge2DOFPrior::computeError() {
    //Eigen::Isometry3d delta=_inverseMeasurement * _cache->n2w();
    const Vertex2DOF* calib  = static_cast<const Vertex2DOF*>(_vertices[0]);
    _error = calib->estimate() - _measurement;
  }


  bool MyEdge2DOFPrior::setMeasurementFromState(){
    const Vertex2DOF* calib  = static_cast<const Vertex2DOF*>(_vertices[0]);
    setMeasurement(calib->estimate());
    return true;
  }


  void MyEdge2DOFPrior::initialEstimate(const OptimizableGraph::VertexSet& /*from_*/, OptimizableGraph::Vertex* /*to_*/) {
    Vertex2DOF *v = static_cast<Vertex2DOF*>(_vertices[0]);
    assert(v && "Vertex for the Prior edge is not set");

    v->setEstimate(_measurement);
  }

}
