#include "g2o_edge_2DOF_mirror_model.h"
#include "g2o/core/factory.h"
#include "g2o/stuff/macros.h"
namespace g2o {

  Edge2DOFMirrorModel::Edge2DOFMirrorModel() :
    BaseMultiEdge<6, Vector6d>()
  {
    resize(3);
  }

  Edge2DOFMirrorModel::Edge2DOFMirrorModel(VertexSE3* cam1,VertexSE3* cam2, Vertex2DOF* calib) :
    BaseMultiEdge<6, Vector6d>()
  {
    resize(3);
    _vertices[0] = cam1;
    _vertices[1] = cam2;
    _vertices[2] = calib;
  }

  bool Edge2DOFMirrorModel::read(std::istream& is)
  {
    for ( int i=0; i<information().rows() && is.good(); i++){
      for (int j=i; j<information().cols() && is.good(); j++){
        is >> information()(i,j);
        if (i!=j)
          information()(j,i)=information()(i,j);
      }
    }
    if (is.bad()) {
      // we overwrite the information matrix with the Identity
      information().setIdentity();
    }
    return true;
  }

  bool Edge2DOFMirrorModel::write(std::ostream& os) const
  {
    for (int i=0; i<information().rows(); i++){
      for (int j=i; j<information().cols(); j++) {
        os << information()(i,j) << " ";
      }
    }
    return os.good();
  }

  void Edge2DOFMirrorModel::computeError() {
    //Vertice of Camera 1
    const VertexSE3* cam1  = static_cast<const VertexSE3*>(_vertices[0]);
    //Vertice of Camera 2
    const VertexSE3* cam2  = static_cast<const VertexSE3*>(_vertices[1]);
    const Vertex2DOF* calib2DOF = static_cast<const Vertex2DOF*>(_vertices[2]);
    
    Eigen::Isometry3d se3_calib = calib2DOF->toIsometry();

    Eigen::Isometry3d delta = ((cam1->estimate() * se3_calib) * cam2->estimate().inverse());
    _error=internal::toVectorMQT(delta);

  
  }


  G2O_REGISTER_TYPE(EDGE_2DOF_MIRROR_MODEL, Edge2DOFMirrorModel);
} // end namespace
