#ifndef EDGE_TRIPLE_SE3_H
#define EDGE_TRIPLE_SE3_H

#include "g2o/types/slam3d/vertex_se3.h"
#include "g2o/core/base_multi_edge.h"

namespace g2o {

  /**
   * \brief Connection between two camera positions and their extrinsic calibration data
   */
  class EdgeTripleSE3: public BaseMultiEdge<6, Eigen::Isometry3d>
  {
    public:
      EIGEN_MAKE_ALIGNED_OPERATOR_NEW
      EdgeTripleSE3();
      EdgeTripleSE3(VertexSE3* cam1,VertexSE3* cam2, VertexSE3* calib);
      typedef VertexSE3 calibration_vertex_type;

      void computeError()
      {
        //Vertice of Camera 1
        const VertexSE3* cam1  = static_cast<const VertexSE3*>(_vertices[0]);
        //Vertice of Camera 2
        const VertexSE3* cam2  = static_cast<const VertexSE3*>(_vertices[1]);
        //Transform between Cameras (from 1 to 2)
        const VertexSE3* calib = static_cast<const VertexSE3*>(_vertices[2]);

        //Abbreviations
        const Eigen::Isometry3d& cam1Tf = cam1->estimate();
        const Eigen::Isometry3d& cam2Tf = cam2->estimate();
        const Eigen::Isometry3d& calibTf = calib->estimate();

        //Eigen::Isometry3d delta = (cam1Tf * calibTf * _measurement * calibTf.inverse() * cam2Tf.inverse());
        Eigen::Isometry3d delta = (cam1Tf * calibTf * _measurement * calibTf.inverse() * cam2Tf.inverse());
        _error=internal::toVectorMQT(delta);
      }

      virtual bool read(std::istream& is);
      virtual bool write(std::ostream& os) const;

  };

}

#endif
