#ifndef RGBDSLAM_EDGE_PRIOR_SE3_H
#define RGBDSLAM_EDGE_PRIOR_SE3_H

#include <g2o/types/slam3d/vertex_se3.h>
#include <g2o/types/slam2d/vertex_se2.h>
#include <g2o/core/base_multi_edge.h>
#include <cmath>//abs
typedef Eigen::Matrix<double, 6,1> Vector6d;
namespace g2o {

  /**
   * \brief Connection between two camera positions and their extrinsic calibration data
   */
  class EdgePriorSE3: public BaseUnaryEdge<6, Vector6d, VertexSE3>
  {
    public:
      EIGEN_MAKE_ALIGNED_OPERATOR_NEW
      EdgePriorSE3();
      EdgePriorSE3(VertexSE3* cam1,VertexSE3* cam2, VertexSE2* calib);

      void computeError()
      {
        //Vertice of Camera 1
        const VertexSE3* cam1  = static_cast<const VertexSE3*>(_vertices[0]);
        //Vertice of Camera 2
        const VertexSE3* cam2  = static_cast<const VertexSE3*>(_vertices[1]);
        //Relative transformation between cams in the plane orthogonal to mirror intersection
        const VertexSE2* se2_calib = static_cast<const VertexSE2*>(_vertices[2]);

        Eigen::AngleAxisd calibration_pitch_aa(se2_calib->estimate().rotation().angle(), Vector3d::UnitX());
        Eigen::Isometry3d calibration_pitch(calibration_pitch_aa);

        //Careful: se2's x is se3's z (optical) axis
        Eigen::Vector3d calibration_translation_vec(0.0,
                                                    se2_calib->estimate().translation().y(), 
                                                    se2_calib->estimate().translation().x());
        Eigen::Translation3d calibration_translation(calibration_translation_vec);
        Eigen::Isometry3d se3_calib = calibration_translation * calibration_pitch;

        Eigen::Isometry3d delta = ((cam1->estimate() * se3_calib) * cam2->estimate().inverse());
        _error=internal::toVectorMQT(delta);

      
      }

      virtual bool read(std::istream& is);
      virtual bool write(std::ostream& os) const;

  };

}

#endif
