#include "g2o_vertex_2DOF.h"
#include <typeinfo>
#include "g2o/core/factory.h"
#include "g2o/stuff/macros.h"

namespace g2o {

  Vertex2DOF::Vertex2DOF() :
    BaseVertex<2, Eigen::Vector2d>()
  {
  }

  bool Vertex2DOF::read(std::istream& is)
  {
    Eigen::Vector2d p;
    is >> p[0] >> p[1];
    setEstimate(p);
    return true;
  }

  bool Vertex2DOF::write(std::ostream& os) const
  {
    Eigen::Vector2d p = estimate();
    os << p[0] << " " << p[1];
    return os.good();
  }

  //Quaternion representing the rotation around X axis
  Eigen::Quaterniond Vertex2DOF::rotationQuat() const
  {
    Eigen::AngleAxisd aa(estimate()[1], Vector3d::UnitX());
    return Eigen::Quaterniond(aa);
  }

  Eigen::Vector3d Vertex2DOF::translationVec() const
  {
    double angle = estimate()[1]; 
    double radius = estimate()[0]; 
    Eigen::Vector3d result(0.0, radius * sin(angle), radius - cos(angle) * radius);
    return result;
  }

  g2o::Vector7d Vertex2DOF::toVectorQuat() const
  {
    Eigen::Quaterniond q = rotationQuat();
    Eigen::Vector3d v = translationVec();

    g2o::Vector7d result;
    result <<  v.x(), v.y(), v.z(), q.x(), q.y(), q.z(), q.w();
    return result;
  }


  Eigen::Isometry3d Vertex2DOF::toIsometry() const
  {
    Eigen::Isometry3d calibration_pitch(this->rotationQuat());
    Eigen::Translation3d calibration_translation(translationVec());
    return calibration_translation * calibration_pitch;
  }

  G2O_REGISTER_TYPE(VERTEX2DOF, Vertex2DOF);

} // end namespace


