#ifndef RGBDSLAM_G2O_VERTEX_2DOF_H
#define RGBDSLAM_G2O_VERTEX_2DOF_H

#include "g2o/config.h"
#include "g2o/core/base_vertex.h"
#include "g2o/stuff/misc.h"
#include <g2o/types/slam3d/isometry3d_mappings.h>

namespace g2o {

  /**
   * \brief Represents 1D translation plus 1D rotation, (x,theta)
   */
  class Vertex2DOF : public BaseVertex<2, Eigen::Vector2d>
  {
    public:
      EIGEN_MAKE_ALIGNED_OPERATOR_NEW
      Vertex2DOF();

      virtual void setToOriginImpl() {
        _estimate = Eigen::Vector2d();
      }

      virtual void oplusImpl(const double* update)
      {
        _estimate[0] += update[0];
        _estimate[1] = normalize_theta(_estimate[1] + update[1]);
      }

      virtual bool setEstimateDataImpl(const double* est){
        _estimate=Eigen::Vector2d(est[0], est[1]);
        return true;
      }

      virtual bool getEstimateData(double* est) const {
        Eigen::Map<Eigen::Vector2d> v(est);
        v = _estimate;
        return true;
      }
      
      virtual int estimateDimension() const { return 2; }

      virtual bool setMinimalEstimateDataImpl(const double* est){
        return setEstimateData(est);
      }

      virtual bool getMinimalEstimateData(double* est) const {
        return getEstimateData(est);
      }
      
      virtual int minimalEstimateDimension() const { return 2; }

      virtual bool read(std::istream& is);
      virtual bool write(std::ostream& os) const;

      g2o::Vector7d toVectorQuat() const;
      Eigen::Quaterniond rotationQuat() const;
      Eigen::Vector3d translationVec() const;
      Eigen::Isometry3d toIsometry() const;
  };

} // end namespace

#endif
